var path = require('path');
var fs = require('fs');
var src = path.join(path.dirname(fs.realpathSync(__filename)), '../src');

var TimeTicks = require(src + '/time-ticks');

var ATU = require(src + '/atu');

var serialport = require('serialport');
var SerialPort = serialport.SerialPort;

function autoDetect(searchForATUX, comName, callback){
  console.log('Scanning', comName, 'for', searchForATUX ? 'ATUX' : 'ATU 8 or 9');
  var timeout, found = null;

  var nextAutoDetect = function(){
    clearTimeout(timeout);
    if(!found && searchForATUX)
      autoDetect(false, comName, callback);
    else
      callback(comName, found);
  };

  var serialPort = new SerialPort(comName, {
    baudrate: (searchForATUX ? 57600 : 9600),
    dataBits: 8,
    stopBits: 1,
    parity: 'none',
    parser: serialport.parsers.readline(new RegExp(ATU.EOT + '|' + ATU.CR), 'BINARY')
  }, false);

  serialPort.open(function(error){

    if(error){
      console.log('ATU auto detect error', error);
      nextAutoDetect();
    } else {

      timeout = setTimeout(function(){
        console.log('Scanning timeout');
        serialPort.close(function(error){
          nextAutoDetect();
        });
      }, 4500);

      serialPort.on('data', function(data) {
        console.log('Auto detect data:', data);

        if(data.substring(1, 3) === 'RT'){
          found = 'atux';
          serialPort.close(function(error){ nextAutoDetect(); });
        }

        if(data.indexOf(ATU.SOH + 'I' + ATU.SP + 'F' + ATU.STX) > -1){
          found = 'atu8';
          serialPort.write('P', function() {
            serialPort.drain(function(){
              serialPort.write('S', function(){
                serialPort.drain(function(){
                  serialPort.close(function(error){ nextAutoDetect(); });
                });
              });
            });
          });
        }

        if(data.indexOf(ATU.SOH + '@' + ATU.SP + 'F' + ATU.STX) > -1){
          found = 'atu9';
          serialPort.write('P', function() {
            serialPort.drain(function(){
              serialPort.write('S', function(){
                serialPort.drain(function(){
                  serialPort.close(function(error){ nextAutoDetect(); });
                });
              });
            });
          });

        }
      });

      ATU.setup(serialPort, searchForATUX ? 'atux' : 'atu9');
    }
  });
}

module.exports = function atuAutoDetect(callback){
  serialport.list(function (err, ports) {
    var comPorts = [];
    ports.forEach(function(port) {
      if(port.comName && port.comName.indexOf('Bluetooth') === -1)
        comPorts.push(port.comName);
    });

    console.log('Detected comports: ', comPorts);

    if(comPorts.length){
      var index = -1;

      var iterator = function(comName, found){
        if(found)
          callback(found, comName);
        else {
          index = index + 1;
          if(comPorts[index])
            autoDetect(true, comPorts[index], iterator);
          else
            console.log('No ATU found');
        }
      };
      iterator(comPorts[index], false);
    } else {
      console.log('No comports available');
    }
});
};

