var path = require('path');
var fs = require('fs');
var src = path.join(path.dirname(fs.realpathSync(__filename)), '../src');

var TimeTicks = require(src + '/time-ticks');

var ATU = {
  SOH: String.fromCharCode(1),
  STX: String.fromCharCode(2),
  EOT: String.fromCharCode(4),
  BS: String.fromCharCode(8),
  LF: String.fromCharCode(10),
  CR: String.fromCharCode(13),
  SP: String.fromCharCode(32),
  SPC: String.fromCharCode(32),
  ESC: String.fromCharCode(27),

  daytime: function(){
    return TimeTicks.ticksToTime(Math.floor(((new Date()).getTime() / 1000)) * 1000);
  },

  parse: function(data) {
    var time, timeTicks, event, node;

    try {
      if(data.substring(1, 3) === 'RT'){
        time = data.substring(4, 16);
        timeTicks = TimeTicks.timeToTicks(time);
        event = { type: 'runningTime', payload: { time: time, ticks: timeTicks }};
      }
      else if(data.substring(1, 8) === 'TRIGGER'){
        var parts = data.split(';');
        node = parts[1];
        var rssi = parts[2];
        var batteryLevel = parts[3];
        var resends = parts[4];
        time = parts[5];
        timeTicks = TimeTicks.timeToTicks(parts[5]);
        event = { type: 'pulse', payload: { timeTicks: timeTicks, node: node, resends: resends, batteryLevel: batteryLevel, RSSI: rssi }};
      } else if(data.indexOf(ATU.SOH + 'I' + ATU.SP + 'F' + ATU.STX) > -1 || data.indexOf(ATU.SOH + '@' + ATU.SP + 'F' + ATU.STX) > -1){
        time = data.substring(8, 19) + '0';
        timeTicks = TimeTicks.timeToTicks(time);
        event = { type: 'runningTime', payload: { time: time, ticks: timeTicks }};
      } else if(data.substring(0, 1) === ATU.SOH && data.indexOf(ATU.SP + 'F' + ATU.STX) > -1){
        node = data.substring(1, 2);
        time = data.substring(8, 20);
        timeTicks = TimeTicks.timeToTicks(time);
        event = { type: 'pulse', payload: { timeTicks: timeTicks, node: node }};
      }
    } catch(error) {
      console.log('Exception parsing ATU data: ', data, ' error:', error);
    }

    return event;
  },

  command: function(command, value){
    return ATU.SOH + command + ATU.STX + value + ATU.EOT;
  },

  trigger: function(serialPort, node){
    if(ATU.version === 'atux')
      serialPort.write(ATU.command('TRIGGER', node), function(err, results){
        console.log('TRIGGER ' + node, err ? 'ERROR' : 'OK', 'bytes', results);
      });
    else
      serialPort.write(node, function(err, results){
        console.log('TRIGGER ' + node, err ? 'ERROR' : 'OK', 'bytes', results);
      });
  },

  setup: function(serialPort, version){
    ATU.version = version;

    serialPort.flush(function(err, results){
      console.log('Flush', err ? 'ERROR' : 'OK', 'bytes', results);
      if(version === 'atux'){
        serialPort.write(ATU.SOH + 'ZZZ' + ATU.STX + ATU.SPC + ATU.SPC + ATU.SPC + ATU.daytime() + ATU.EOT, function(err, results){
          console.log('Set daytime', err ? 'ERROR' : 'OK', 'bytes', results);
          serialPort.drain(function(){
            serialPort.write(ATU.command('RUNNINGTIME', '1'), function(err, results){
              console.log('Running time', err ? 'ERROR' : 'OK', 'bytes', results);
            });
          });
        });
      } else {
        serialPort.write('S', function() {
          serialPort.drain(function(){
            serialPort.write('R', function() {
              serialPort.drain(function(){
                serialPort.write('V', function(){
                  serialPort.drain(function(){
                    serialPort.write(ATU.SOH + 'ZZZ' + ATU.STX + ATU.SPC + ATU.SPC + ATU.SPC + ATU.daytime() + ATU.EOT, function(err, results){
                      console.log('Set daytime', err ? 'ERROR' : 'OK', 'bytes', results);
                    });
                  });
                });
              });
            });
          });
        });
      }
    });
  }
};

module.exports = ATU;
