var path = require('path');
var fs = require('fs');
var src = path.join(path.dirname(fs.realpathSync(__filename)), '../src');
var TimeTicks = require(src + '/time-ticks');
var jumping = require(src + '/jumping');
var udpServer = require(src + '/udp-server');
var resultProvider = require(src + '/results-provider');

global.outputData = {};
global.results = [];
global.xmlResultProviderUrl = null;
global.lastUdpMessage = '';

var Output = {

  STOP_FENCE_FAULTS: {
    555: true,
    666: true,
    777: true,
    888: true,
    999: true,
    998: true
  },

  REASON: {
    A: 'WD',
    U: 'RET',
    D: 'EL',
    S: 'SUSP'
  },

  socketBroadcast: function(serv, eventData) {
    var message = JSON.stringify(eventData);
    serv.connections.forEach(function (conn) {
      conn.sendText(message);
    });
  },

  udpBroadcast: function(xml){
    udpServer.broadcast(xml);
  },

  // updateOutputData: function(event){
  //   if(global.outputData.countDown && !global.outputData.running){
  //     global.outputData.countDownValue = TimeTicks.fixedTo(Number((global.outputData.countDownValue * 10 + 1) + 'e-1'), 1);
  //     if(global.outputData.countDownValue.lastIndexOf('.0') !== -1)
  //       Output.udpBroadcast(jumping(global.outputData));
  //   }
  //   else if(global.outputData.running){
  //     global.outputData.time = TimeTicks.fixedTo(Number((global.outputData.time * 10 + 1) + 'e-1'), 1);
  //     if(global.outputData.time.lastIndexOf('.0') !== -1)
  //       Output.udpBroadcast(jumping(global.outputData));
  //   }
  // },

  updateOutputData: function(event){
    var t;
    if(global.outputData.countDown && !global.outputData.running){
      t = Number((global.outputData.countDownValue * 10 + 1) + 'e-1');
      global.outputData.countDownValue = TimeTicks.fixedTo(t, 1);
      console.log(global.outputData.countDownValue);
      if((t * 10) % 10 === 0){
        Output.udpBroadcast(jumping(global.outputData));
      }
    }
    else if(global.outputData.running){
      t = Number((global.outputData.time * 10 + 1) + 'e-1');
      global.outputData.time = TimeTicks.fixedTo(t, 1);
      if((t * 10) % 10 === 0)
        Output.udpBroadcast(jumping(global.outputData));
    }
  },

  decorateBroadcast: function(){
    global.results.forEach(function (start) {
      if(start.id === global.outputData.id){
        global.outputData.horseNo = start.horse_no;
        global.outputData.riderName = start.rider_name;
        global.outputData.riderCountry = start.rider_country;
        global.outputData.clubName = start.club_name;
        global.outputData.horseName = start.horse_name;
        global.outputData.logoId = start.logo_id;
        global.outputData.logoGroup = start.logo_group;
        if(!Output.REASON[start.or])
          global.outputData.rank = start.re;
        global.outputData.resultPreview = start.result_preview;
        global.outputData.status = Output.REASON[start.or] || '';

        console.log('Update global data ', global.outputData.id, ' ', start.rider_name, ' ', 'rank:', start.re);
      }
    });
    Output.udpBroadcast(jumping(global.outputData));
  },

  storeOutputData: function(payload){
    var data = payload;

    if(Output.STOP_FENCE_FAULTS[payload.fenceFaults]){
      if(data.running)
        data.running = false;

      data.faults = '';
      data.timeFaults = '';
    }

    if(!data.countDown && global.outputData.countDown && !data.running){
      global.outputData.countDown = false;
      global.outputData.faults = data.countDownValue;
      Output.udpBroadcast(jumping(global.outputData));
    }

    data.faults = data.faults || 0;
    data.timeFaults = data.timeFaults || 0;
    data.time = data.time || 0;

    if(data.countDown && data.countDownValue)
      data.countDownValue = parseFloat(data.countDownValue, 10) || -45.0;

    if(!data.running)
      data.time = TimeTicks.fixedTo(parseFloat(data.time, 10) || 0, 2);

    global.outputData = data;

    Output.decorateBroadcast(udpServer);
  },

  storeResults: function(results){
    global.results = results;
    Output.decorateBroadcast();
    if(global.xmlResultProviderUrl)
      resultProvider.load(global.xmlResultProviderUrl);
    global.xmlResultProviderUrl = null;
  },

  fetchResultProviderOnNextResult: function(url){
    global.xmlResultProviderUrl = url;
  }

};

module.exports = Output;
