#!/usr/bin/env node

console.log('Equipe ATU Socket');
console.log('Install USB drivers for the ATU and connect it to the computer before you start this application');

var path = require('path');
var fs = require('fs');
var src = path.join(path.dirname(fs.realpathSync(__filename)), '../src');
var ATU = require(src + '/atu');
var ws = require('nodejs-websocket');
var atuAutoDetect = require(src + '/atu-auto-detect');
var serialport = require('serialport');
var SerialPort = serialport.SerialPort;
var serialPort;
var output = require(src + '/output');

global.atuVersion = 'atux';

var server = ws.createServer(function (conn) {
    console.log('New connection');
    conn.on('text', function (jsonStr) {
      var data = JSON.parse(jsonStr);
      if(data.type === 'cmd'){
        if(data.payload.cmd === 'trigger')
          ATU.trigger(serialPort, data.payload.node);
      } else if(data.type === 'output'){ /* Toggle this to enable/disable */
        output.storeOutputData(data.payload);
      } else if(data.type === 'results'){
        output.storeResults(data.payload);
      } else if(data.type === 'xmlResultProviderUrl'){
        console.log('Pending XmlResultProviderUrl');
        output.fetchResultProviderOnNextResult(data.payload);
      }
    });
    conn.on('close', function (code, reason) {
      console.log('Connection closed');
    });
    conn.on('error', function (error) {
      console.log('Websocket errror');
    });
}).listen(8080, function(data){
  console.log('Websocket Server listening on ws://localhost:8080 (Use this address for app.equipe.com)');
});


atuAutoDetect(function(version, comName){
  console.log('Auto detected', version.toUpperCase(), '@', comName);

  serialPort = new SerialPort(comName, {
    baudrate: (version === 'atux' ? 57600 : 9600),
    dataBits: 8,
    stopBits: 1,
    parity: 'none',
    parser: serialport.parsers.readline(ATU.EOT, 'BINARY')
  });

  serialPort.on('open', function () {
    ATU.setup(serialPort, version);

    serialPort.on('data', function(data) {
      var event = ATU.parse(data);

      if(event){
        output.socketBroadcast(server, event);

        if(event.type === 'runningTime')
          output.updateOutputData(event);

        console.log(event);
      }

    });

  });

  serialPort.on('close', function(){
    console.log('Comport close');
  });

  serialPort.on('error', function(error) {
    console.log('Error', error);
  });

});
