var connected = {};

var iconv = require('iconv-lite');
var dgram = require('dgram');
var udp = dgram.createSocket('udp4');
var zlib = require('zlib');

udp.on('listening', function () {
  var address = udp.address();
  console.log('UDP Server listening on ' + address.address + ':' + address.port + ' (Use The Pilot to connect)');
});

udp.on('message', function (data, remote) {
  var message = data.toString('utf8');
  if(message === 'QUIT')
    delete connected[remote.address];
  else
    if(message === 'CONNECTED' || message === 'PING'){
      connected[remote.address] = {
        port: remote.port,
        lastSeen: Date.now()
      };
      var response = new Buffer('OK');
      udp.send(response, 0, response.length, remote.port, remote.address);
    }
  console.log('"', message + '', '"');
  console.log(connected);
  console.log(remote.address + ':' + remote.port + ' - ' + message);
});

udp.bind(8005);

var udpServer = {
  connections: function(){
    return connected;
  },

  hasConnections: function(){
    return Object.keys(connected).length;
  },

  startCleaner: function(){
    if(udpServer.cleanerInterval)
      clearInterval(udpServer.cleanerInterval);
    udpServer.cleanerInterval = setInterval(udpServer.cleaner, 1.5 * 60 * 1000);
  },

  cleaner: function() {
    Object.keys(connected).forEach(function(key) {
      var val = connected[key];
      if(val.lastSeen > (Date.now() + 1.5 * 60 * 1000)){
        console.log('Removed inactive client ' + key);
        delete connected[key];
      }
    });
  },

  broadcast: function(data) {
    var message = iconv.encode(data, 'iso-8859-1');

    zlib.deflate(message, function(err, buffer) {
      if (!err) {

        Object.keys(connected).forEach(function(key) {
          var val = connected[key];

          udp.send(buffer, 0, buffer.length, val.port, key, function(udpError, bytes) {
            if(udpError)
              console.log('UDP error ' + key + ':' + val.port, udpError);
          });

        });


      } else {
        console.log('Error deflate');
      }
    });

  }
};

udpServer.startCleaner();

module.exports = udpServer;
