#!/usr/bin/env node

console.log('Testing mode');

var path = require('path');
var fs = require('fs');
var src = path.join(path.dirname(fs.realpathSync(__filename)), '../src');
var ws = require('nodejs-websocket');
var output = require(src + '/output');
var TimeTicks = require(src + '/time-ticks');

global.atuVersion = 'atux';

var buildPulseEvent = function(node){
  var ticks = (new Date()).getTime();
  return {
    type: 'pulse',
    payload: {
      node: node,
      timeTicks: ticks
    }
  };
};

var server = ws.createServer(function (conn) {
    conn.on('text', function (jsonStr) {
      var data = JSON.parse(jsonStr);
      if(data.type === 'cmd'){
        if(data.payload.cmd === 'trigger')
          output.socketBroadcast(server, buildPulseEvent(data.payload.node));
      } else if(data.type === 'output'){
        output.storeOutputData(data.payload);
      } else if(data.type === 'results'){
        output.storeResults(data.payload);
      } else if(data.type === 'xmlResultProviderUrl'){
        console.log('Pending XmlResultProviderUrl');
        output.fetchResultProviderOnNextResult(data.payload);
      }
    });
    conn.on('close', function (code, reason) {});
    conn.on('error', function (error) {});
}).listen(8080, function(data){
  console.log('Websocket Server listening on ws://localhost:8080 (Use this address for app.equipe.com)');
});

var runningTime = function(){
  var ticks = (new Date()).getTime();
  var event = {
    type: 'runningTime',
    payload: {
      time: TimeTicks.ticksToTime(ticks),
      ticks: ticks
    }
  };
  output.socketBroadcast(server, event);
  output.updateOutputData(event);
};

setInterval(runningTime, 100);


